#ifndef DEMOAPP_H
#define DEMOAPP_H

#include "DemoDlgImp.h"
#include "OOV.h"
#include <fmod.h>

class DemoApp : public QApplication {
    Q_OBJECT
public:
    DemoApp(int argc, char* argv[]);
    virtual ~DemoApp();

    void createLocalLabels();

    void displayRecord();
    void displayPosteriors();
    void displayRecognizedLabels();

    unsigned int screenWidth(double f=1) const;
    unsigned int screenHeight(double f=1) const;

    unsigned int contextFrames() const;
    void setContextFrames(unsigned int=75);

    OOV::Classification classifyLabel(const OOV::Label& l);
    void classify();

protected:
    // initialize some records,lists,...
    void init();

    // words that can be skipped for the SRover aligned rec
    bool skippableAlignWord(OOV::Label l);

    // loads the sound for the currently active record into memory
    void loadSound();

    // transfers points from one rect into another rect
    QPointF winToWin(const QPointF& p, const QRectF& r1, const QRectF& r2) const;

    //! get back one of the sets of class icons!
    const QIcon& getClassIcon(OOV::Classification c);

    //! get color from Class
    QColor classToColor(OOV::Classification);

    DemoDlgImp* mDemoDlg;
    OOV::RecordSet* mRecordSet;

protected slots:
    void onDictionaryTextChanged(const QString&);
    void onPronTextChanged(const QString&);

    void onNextPressed();
    void onPrevPressed();

    void onPlayUttPressed();
    void onPlayWinPressed();

    void stopPlayback();

    void recWordSelected(QListWidgetItem*);

    void onContextValueChanged(int);

    void update();
    void updateClassification();

private:
    QList<OOV::Label> mAlignRecs;
    QMap<QString,unsigned int> mPhones;

    QRect mStrongPostWin;
    QRect mWeakPostWin;
    unsigned int mContextFrames;

    QTimer mContextTimer;

    FMOD_SYSTEM* mpFmodSystem;
    FMOD_SOUND* mpFmodSound;
    FMOD_CHANNEL* mpFmodChannel;
    unsigned int mFmodVersion;
};

#endif
