#include "DemoDlgImp.h"
#include <QSpinBox>

#include <QtGlobal>

DemoDlgImp::DemoDlgImp(QWidget * parent, Qt::WFlags f) : QDialog(parent, f) {
    setupUi(this);

    mThresholds.resize(OOV::CLASS_MAX);


    connect(quitPB,SIGNAL(clicked()),qApp,SLOT(quit()));

    connect(classificationCoB,SIGNAL(activated(int)),this,SLOT(onTaskChanged(int)));
    connect(classificationCoB,SIGNAL(activated(int)),classificationStack,SLOT(setCurrentIndex(int)));
    connect(triangleInput,SIGNAL(priorsChanged(double,double,double)),this,SIGNAL(valueChanged()));


    // registering controls...
    mControlMapper=new QSignalMapper(this);
    connect(mControlMapper,SIGNAL(mapped(QWidget*)),this,SLOT(onValueChanged(QWidget*)));

    // for OOV/non-OOV task
    registerControl(SB_2,S_2,OOV::CLASS_OOV);
    // for CORR/MISREC task
    registerControl(SB_3,S_3,OOV::CLASS_Corr);
    // for OOV/IVINCORR|MISREC task
    registerControl(SB_4,S_4,OOV::CLASS_OOV);

}



OOV::Task DemoDlgImp::activeTask() const {
    int t=classificationCoB->currentIndex();
    if ((t<0)||(t>=OOV::TASK_MAX)) { qDebug()<<"test is not known!"; return OOV::TASK_FullClassification; }
    else { /*qDebug()<<t;*/ return (OOV::Task)t; }
}

void DemoDlgImp::onTaskChanged(int p){
//    qDebug()<<"TASK "<<p;
    emit taskChanged((OOV::Task)p);
}

void DemoDlgImp::onValueChanged(QWidget* obj) {
    //qDebug()<<"valueChanged";

    QSlider* srcs=dynamic_cast<QSlider*>(obj);
    if (srcs) {
        //qDebug()<<"for qslider";
        const int v=srcs->value();

        QDoubleSpinBox* tgt=dynamic_cast<QDoubleSpinBox*>(mSrcToTgt[obj]);

        if (!tgt) return;

        const int f=srcs->maximum()-srcs->minimum();
        double& t=mThresholds[mObjToClass[obj]];
        t=((double)v)/f;

        if (tgt->value()!=t) {
            tgt->setValue(t);
        }
    };

    QDoubleSpinBox* srcsl=dynamic_cast<QDoubleSpinBox*>(obj);
    if (srcsl) {
        //qDebug()<<"for qdoublespinbox";

        double &v=mThresholds[mObjToClass[obj]];

        v=srcsl->value();

        QSlider* tgt=dynamic_cast<QSlider*>(mSrcToTgt[obj]);

        if (!tgt) return;

        const int f=tgt->maximum()-tgt->minimum();

        if (tgt->value()!=qRound(v*f)) {
            tgt->setValue(qRound(v*f));
        }
    }

    if (!srcsl&&!srcs) { qDebug()<<"ERROR!Wrong obj!"; };
    emit valueChanged();
}

void DemoDlgImp::registerControl(QDoubleSpinBox* sb,QSlider* s,OOV::Classification c) {
    qDebug()<<"registerControl";
    mSrcToTgt[sb]=s;
    mSrcToTgt[s]=sb;
    mObjToClass[sb]=mObjToClass[s]=c;

    connect(s,SIGNAL(valueChanged(int)),mControlMapper,SLOT(map()));
    mControlMapper->setMapping(s,s);
    connect(sb,SIGNAL(valueChanged(double)),mControlMapper,SLOT(map()));
    mControlMapper->setMapping(sb,sb);
}


