#include "OOV.h"

#include <QtGlobal>

OOV::RecordSet::RecordSet(QObject* parent) : QObject(parent) {
/*    setDataPath("./data");
    qDebug() << "OOV::RecordSet";

    mOOVEval.resize(EVAL_MAX);
    mMisrecEval.resize(EVAL_MAX);
    mSilEval.resize(EVAL_MAX);*/
}

OOV::RecordSet::~RecordSet() {
    qDebug() << "OOV::~RecordSet";
}


bool OOV::RecordSet::loadDictionary(const QString& dctFN) {
    //! setup OOV dictionary
    QFile dctFile(dataPath()+dctFN);

    if (!dctFile.open(QIODevice::ReadOnly|QIODevice::Text)) {
        qDebug() << "ERROR! Dictionary not readable!";
        return false;
    }

    QTextStream in(&dctFile);

    QString word, phonestr;
    QStringList wrdlst;

    mDictionary.clear();
    while (!in.atEnd()) {
        in >> word >> phonestr;
        if (!word.isEmpty())
        {
            mDictionary[word]=phonestr;
            wrdlst << word;
        }
    }
    mDictionary["!NULL"]="";
    mDictionary[""]="";
    mDictionary["<pau>"]="sil";
    mDictionary["<PAU>"]="sil";

    return true;
}


QMap <QString,QString> OOV::RecordSet::dictionary() const {
    return mDictionary;
}


void OOV::RecordSet::setDataPath(const QString& p) {
    mDataPath=p;
    if (!p.endsWith('/')) mDataPath.append('/');
}

const QString& OOV::RecordSet::dataPath() const {
    return mDataPath;
}

void OOV::RecordSet::setActiveFrame(int a) {
    int ar=activeRecord();
    if (ar<0) return;
    mRecordSet[*activeRecordName()].activeFrame=a;
}

int OOV::RecordSet::activeFrame() const {
    int ar=activeRecord();
    if (ar<0) return -1;
    return mRecordSet[*activeRecordName()].activeFrame;
}


bool OOV::RecordSet::loadFrameScores(const QString& wildcard) {
    //qDebug() << mRecordName.count() << " RECORD NAMES";
    for (int i=0;i<mRecordName.count();i++) {
    
        QList <Score> scores;
        Score score;

        QString fn = dataPath()+wildcard.arg(mRecordName[i]);

        QFile f(fn);

        if (!f.open(QIODevice::ReadOnly|QIODevice::Text)) {
            qDebug() << fn << "FAILED TO OPEN!";
            return false;
        } 

        QTextStream in(&f);

        QString sil, ivcorr, ivincorr, oov;

        in >> ivcorr;

        while (!in.atEnd()){
            bool b1,b2,b3,b4;
            in >> ivincorr >> oov >> sil;
            score.sil=sil.toFloat(&b1);
            score.ivcorr=ivcorr.toFloat(&b2);
            score.ivincorr=ivincorr.toFloat(&b3);
            score.oov=oov.toFloat(&b4);

            if (!(b1&&b2&&b3&&b4)) {
                scores.clear();
                qDebug() << "YUUUCK! SOMETHING FAILED!";
                return false;
            }
            scores.append(score);
            in >> ivcorr;
        }

        //qDebug() << fn << "SUCEEDED";

        mRecordSet[mRecordName[i]].scores = scores;
    }
    //qDebug() << "SUCEEDED LOADING _ALL_ SCORES!";
    return true;
}

bool OOV::RecordSet::loadFramePosteriors(const QString& wildcard) {
    //qDebug() << mRecordName.count() << " RECORD NAMES";
    for (int i=0;i<mRecordName.count();i++) {
        mRecordSet[mRecordName[i]].strongposts.load(dataPath()+QString(wildcard).arg(mRecordName[i]).arg("strong"));
        mRecordSet[mRecordName[i]].weakposts.load(dataPath()+QString(wildcard).arg(mRecordName[i]).arg("weak"));
        //qDebug() << dataPath()+QString(wildcard).arg(mRecordName[i]) << "LOADED!";
     }
    return true;
}

bool OOV::RecordSet::loadMLF(const QString& fileName, const QString& hash) {
    Label lab;
    Record rec;

    bool firstLoad=mRecordSet.isEmpty();

    QString fn = dataPath()+fileName;

    QFile f(fn);

    if (!f.open(QIODevice::ReadOnly|QIODevice::Text)) {
        qDebug() << fn << "FAILED TO OPEN!";
        return false;
    }

    QTextStream in(&f);

    QString mlfMark, recName, startTime, stopTime, item;

    in>>mlfMark;
    //qDebug()<<mlfMark;

    // read & clean record name
    in>>recName;
    rec.recordName=recName.remove('"').remove(".lab").remove(".rec").remove(QRegExp("^.*/"));
    rec.audioFile=rec.recordName;
    rec.audioFile.prepend(dataPath()+"audio/");
    rec.audioFile.append(".ogg");

    int r=0;

    while (!in.atEnd()){
        //qDebug()<<"new record:"<<recName;
        r++;
        while (!in.atEnd()) {
            in>>startTime;
            if (startTime=="."){
                // finished reading one MLF record...
                rec.activeFrame=-1;
                //qDebug()<<"end of record!"<<rec.labels[hash].count()<<":"<<r;
                if (firstLoad) {
                    rec.audioFile=rec.recordName;
                    rec.audioFile.prepend(dataPath()+"audio/");
                    rec.audioFile.append(".ogg");
                    mRecordSet[rec.recordName]=rec;
                    mRecordName.append(rec.recordName);
                    //qDebug() << "WRITE "<<rec.labels.count();
                }
                else {
                    if (!mRecordSet.contains(rec.recordName)) {
                        qDebug() << "ERROR! UNEXPECTED RECORD NAME! FAIL!";
                        return false;
                    }
                    mRecordSet[rec.recordName].labels[hash]=rec.labels[hash];
                }
                rec.labels[hash].clear();
                rec.labels.clear();

                // read & clean record name
                in>>recName;
                rec.recordName=recName.remove('"').remove(".lab").remove(".rec").remove(QRegExp("^.*/"));
                break;
            } else {
                in>>stopTime>>item;
                lab.labelName=item;
                lab.start=startTime.toInt()/100000;
                lab.stop=stopTime.toInt()/100000;
                rec.labels[hash].append(lab);
                //qDebug()<<lab.start<<":"<<lab.stop<<":"<<item1<<":"<<item2;
            }
        }
    }

    if (firstLoad) {
        setActiveRecord(0);
    }

    return true;
}


bool OOV::RecordSet::loadAlignedMLF(const QString& fileName, const QString& h1, const QString& h2) {
    Label lab;
    Record rec;

    QString fn=dataPath()+fileName;

    QFile f(fn);

    if (h1==h2) {
        qDebug()<<"BOTH HASHES ARE EQUAL!ERROR!";
        return false;
    }

    if (!f.open(QIODevice::ReadOnly|QIODevice::Text)) {
        qDebug()<<fn<<"FAILED TO OPEN!";
        return false;
    }

    QTextStream in(&f);

    QString mlfMark,recName,startTime,stopTime,item1,item2;

    in>>mlfMark;

    // read & clean record name
    in>>recName;
    rec.recordName=recName.remove('"').remove(".lab").remove(".rec").remove(QRegExp("^.*/"));
    rec.audioFile=rec.recordName;
    rec.audioFile.prepend(dataPath()+"audio/");
    rec.audioFile.append(".ogg");

    int r=0;

    while (!in.atEnd()){
        //qDebug()<<"new record:"<<recName;
        r++;
        while (!in.atEnd()) {
            in>>startTime;
            if (startTime=="."){
                // finished reading one MLF record...
                rec.activeFrame=-1;
                //qDebug()<<"end of record!"<<rec.labels[h1].count()<<":"<<r;
                mRecordSet[rec.recordName]=rec;

                rec.labels[h1].clear(); rec.labels[h2].clear();
                rec.labels.clear();
                mRecordName.append(rec.recordName);

                // read & clean record name
                in>>recName;
                rec.recordName=recName.remove('"').remove(".lab").remove(".rec").remove(QRegExp("^.*/"));
                rec.audioFile=rec.recordName;
                rec.audioFile.prepend(dataPath()+"audio/");
                rec.audioFile.append(".ogg");

                break;
            } else {
                in>>stopTime>>item1>>item2;
                lab.labelName=item1;
                lab.start=startTime.toInt()/100000;
                lab.stop=stopTime.toInt()/100000;
                rec.labels[h1].append(lab);
                lab.labelName=item2;
                rec.labels[h2].append(lab);
                //qDebug()<<lab.start<<":"<<lab.stop<<":"<<item1<<":"<<item2;
            }
        }
    }

    setActiveRecord(0);

    return true;
}


void OOV::RecordSet::setActiveRecord(int rec) {
    mActiveRecord=qMin(rec,mRecordSet.count()-1);
}

void OOV::RecordSet::nextActiveRecord() {
    mActiveRecord++;
    if (mActiveRecord==mRecordSet.count()) mActiveRecord=0;
}

void OOV::RecordSet::prevActiveRecord() {
    mActiveRecord--;
    if (mActiveRecord<0) mActiveRecord=mRecordSet.count()-1;
}

int OOV::RecordSet::activeRecord() const {
    return mActiveRecord;
}

const QString* OOV::RecordSet::activeRecordName() const {
    if (qAbs(activeRecord()) < mRecordName.size())
        return &mRecordName[activeRecord()];
    else return 0;
}


QList <OOV::Label> OOV::RecordSet::getLabels( const QString& hash) const {
    QList <OOV::Label> l;

    if (!activeRecordName()) return l;

    if (mRecordSet[*activeRecordName()].labels.contains(hash))
        return mRecordSet[*activeRecordName()].labels[hash];
    else
        return l;
}

const OOV::Label* OOV::RecordSet::activeLabel(const QString& h) const {
    if (!activeRecordName()) return 0;
    int ix=mRecordSet[*activeRecordName()].activeLabelIndex(h);
    if (ix<0) return 0;
    return &mRecordSet[*activeRecordName()].labels[h][ix];
}



unsigned int OOV::RecordSet::recordCount() const {
    return mRecordSet.count();
}


const QImage* OOV::RecordSet::getStrongFramePosteriors() {
    if (!activeRecordName()) return 0;
    return &mRecordSet[*activeRecordName()].strongposts;
}

const QImage* OOV::RecordSet::getWeakFramePosteriors() {
    if (!activeRecordName()) return 0;
    return &mRecordSet[*activeRecordName()].weakposts;
}

QList<OOV::Score> OOV::RecordSet::getFrameScores() {
    QList<Score> l;
    if (!activeRecordName()) return l;
    else return mRecordSet[*activeRecordName()].scores;
}


QString OOV::RecordSet::activeSound() const {
    QString snd="";
    if (!activeRecordName()) return snd;
    else return mRecordSet[*activeRecordName()].audioFile;
}

unsigned int OOV::RecordSet::frameCount() const {
    if (!activeRecordName()) return 0;
    return mRecordSet[*activeRecordName()].scores.count();
}

void OOV::RecordSet::classify(double sil,double misrec,double oov,const QString& hash) {
    qDebug() << "CLASSIFYING ALL!";
}




