 /****************************************************************************
 **
 ** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
 ** Contact: Qt Software Information (qt-info@nokia.com)
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License versions 2.0 or 3.0 as published by the Free
 ** Software Foundation and appearing in the file LICENSE.GPL included in
 ** the packaging of this file.  Please review the following information
 ** to ensure GNU General Public Licensing requirements will be met:
 ** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
 ** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
 ** exception, Nokia gives you certain additional rights. These rights
 ** are described in the Nokia Qt GPL Exception version 1.3, included in
 ** the file GPL_EXCEPTION.txt in this package.
 **
 ** Qt for Windows(R) Licensees
 ** As a special exception, Nokia, as the sole copyright holder for Qt
 ** Designer, grants users of the Qt/Eclipse Integration plug-in the
 ** right for the Qt/Eclipse Integration to link to functionality
 ** provided by Qt Designer and its related libraries.
 **
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 **
 ****************************************************************************/

#include <QtGui>
#include "triangleinput.h"

TriangleInput::TriangleInput(QWidget *parent) : QWidget(parent) {
    mT<<QPointF(-0.5,0.43)<<QPointF(0.5,0.43)<<QPointF(0,-0.43);
    setPriors(0.33,0.33,0.33);
    connect(this,SIGNAL(priorsChanged(double,double,double)),this,SLOT(update()));
}

TriangleInput::~TriangleInput() {
}

QPointF TriangleInput::winToWin(const QPointF& p, const QRectF& r1, const QRectF& r2) const {
    QPointF r;
    QPointF p1=r1.topLeft();
    QPointF p3=r2.topLeft();

    r.setX((p.x()-p1.x())*r2.width()/r1.width()+p3.x());
    r.setY((p.y()-p1.y())*r2.height()/r1.height()+p3.y());

    return r;
}

void TriangleInput::paintEvent(QPaintEvent*) {
    qDebug()<<mPrior[0]<<":"<<mPrior[1]<<mPrior[2];
    QPen pen;
    pen.setWidth(2);

    int mindim=qMin(width(),height());

    QRectF from(0,0,1,1);
    from.moveCenter(QPointF(0,0));
    QRectF to(0,0,mindim,mindim);
    to.moveCenter(QPointF(width()*0.5,height()*0.5));

    QPointF o=mPrior[0]*mT[0]+mPrior[1]*mT[1]+mPrior[2]*mT[2];

    QPolygonF t,t1,t2,t3;
    t<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);
    t1<<winToWin(o,from,to)<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to);
    t2<<winToWin(o,from,to)<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);
    t3<<winToWin(o,from,to)<<winToWin(mT[2],from,to)<<winToWin(mT[0],from,to);

    QPainter p(this);
    p.setPen(pen);
//    p.setBrush(QBrush(Qt::red));
//    p.drawPolygon(t);

    p.setBrush(QBrush(QColor::fromRgbF(0,0.7,0.3)));
    p.drawPolygon(t1);
    p.setBrush(QBrush(Qt::yellow));
    p.drawPolygon(t2);
    p.setBrush(QBrush(Qt::red));
    p.drawPolygon(t3);

    p.end();
}

void TriangleInput::getPriors(double& p1,double& p2,double& p3) {
    p1=mPrior[0];
    p2=mPrior[1];
    p3=mPrior[2];
}

void TriangleInput::setPriors(double p1,double p2,double p3) {
    // set mPrior based on the triangle/the tri-set of priors
    //! FIXME to be implemented...
    mPrior[0]=p1;mPrior[1]=p2;mPrior[2]=p3;
    emit update();
}

void TriangleInput::mouseMoveEvent(QMouseEvent* ev){
    if (ev->buttons()&&Qt::LeftButton) {
        QPointF p(ev->posF());
        int mindim=qMin(width(),height());

        QRectF from(0,0,1,1);
        from.moveCenter(QPointF(0,0));
        QRectF to(0,0,mindim,mindim);
        to.moveCenter(QPointF(width()*0.5,height()*0.5));

        QPolygonF t;
        t<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);

        if (t.containsPoint(p,Qt::WindingFill)){
            QPolygonF t1,t2,t3;
            t1<<p<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to);
            t2<<p<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);
            t3<<p<<winToWin(mT[2],from,to)<<winToWin(mT[0],from,to);

            double p1,p2,p3;
            mPrior[0]=p1=triangleArea(t2)/triangleArea(t);
            mPrior[1]=p2=triangleArea(t3)/triangleArea(t);
            mPrior[2]=p3=triangleArea(t1)/triangleArea(t);
            emit priorsChanged(p1,p2,p3);
            emit repaint();
        }
    }
}

void TriangleInput::mousePressEvent(QMouseEvent* ev){
    if (ev->button()==Qt::LeftButton) {
        QPointF p(ev->posF());
        int mindim=qMin(width(),height());

        QRectF from(0,0,1,1);
        from.moveCenter(QPointF(0,0));
        QRectF to(0,0,mindim,mindim);
        to.moveCenter(QPointF(width()*0.5,height()*0.5));

        QPolygonF t;
        t<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);

        if (t.containsPoint(p,Qt::WindingFill)){
            QPolygonF t1,t2,t3;
            t1<<p<<winToWin(mT[0],from,to)<<winToWin(mT[1],from,to);
            t2<<p<<winToWin(mT[1],from,to)<<winToWin(mT[2],from,to);
            t3<<p<<winToWin(mT[2],from,to)<<winToWin(mT[0],from,to);

            double p1,p2,p3;
            mPrior[0]=p1=triangleArea(t2)/triangleArea(t);
            mPrior[1]=p2=triangleArea(t3)/triangleArea(t);
            mPrior[2]=p3=triangleArea(t1)/triangleArea(t);
            emit priorsChanged(p1,p2,p3);
        }
    }
}

double TriangleInput::triangleArea(QPolygonF p) const {
    QLineF a(p[0],p[1]);
    QLineF b(p[0],p[2]);
    return sin(b.angleTo(a)/180*3.1415)*a.length()*b.length()*0.5;
}

